const path = require('path')

/**
 * @type {import('@vuepress/types').Theme<import('@vuepress/types').DefaultThemeConfig>}
 */
module.exports = (options, ctx) => {
  const { themeConfig, siteConfig } = ctx

  // resolve algolia
  const isAlgoliaSearch = (
    themeConfig.algolia
    || Object
        .keys(siteConfig.locales && themeConfig.locales || {})
        .some(base => themeConfig.locales[base].algolia)
  )

  const enableSmoothScroll = themeConfig.smoothScroll === true

  let meilisearch = []
  if (options.searchServer && options.searchApiKey && options.searchIndexUid) {
    meilisearch = [
      'vuepress-plugin-meilisearch',
      {
        hostUrl: options.searchServer, // Mandatory
        apiKey: options.searchApiKey, // Mandatory
        indexUid: options.searchIndexUid, // Mandatory
        placeholder: options.searchPlaceholder || '搜索', // Default: ""
        maxSuggestions: 10, // Default: 5
        hotKeys: [], // Default: ['s', '/']
        cropLength: 50, // Default: 30
        layout: 'columns', // Default: "columns", "simple"
        debug: false, // Default: false
        enableDarkMode: false // Default: false
      }
    ]
  }

  return {
    alias () {
      return {
        '@AlgoliaSearchBox': isAlgoliaSearch
          ? path.resolve(__dirname, 'components/AlgoliaSearchBox.vue')
          : path.resolve(__dirname, 'noopModule.js')
      }
    },

    plugins: [
      require('./plugins/auto-sidebar.js'),
      require('./plugins/code-tabs.js'),
      ['@vuepress/active-header-links', options.activeHeaderLinks],
      '@vuepress/search',
      '@vuepress/plugin-nprogress',
      ['smooth-scroll', enableSmoothScroll],
      ['flexsearch-pro'],
      meilisearch,
      ['code-copy', true],
      ['container', {
        type: 'tip',
        defaultTitle: {
          '/': 'TIP',
          '/zh/': '提示'
        }
      }],
      ['container', {
        type: 'warning',
        defaultTitle: {
          '/': 'WARNING',
          '/zh/': '注意'
        }
      }],
      ['container', {
        type: 'danger',
        defaultTitle: {
          '/': 'DANGER',
          '/zh/': '警告'
        }
      }],
      ['container', {
        type: 'info',
        defaultTitle: ' ',
        before: info => `<div class="custom-block info">${info}`,
        after: '</div>',
      }],
      ['container', {
        type: 'details',
        before: info => `<details class="custom-block details">${info ? `<summary>${info}</summary>` : ''}\n`,
        after: () => '</details>\n'
      }]
    ]
  }
}
