/* eslint-env node */
const path = require('path')
const fs= require('fs');
const { convertSidebar } = require('../util/convert')
const { iterateDocs, writeSidebarJs } = require('../util/file')
const chokidar = require('chokidar')

const sourceDir = getSourceDir() //'docs'
const docsRootPath = path.resolve(`./${sourceDir}`)
const vuepressPath = path.resolve(`./${sourceDir}/.vuepress`)

console.log('plugin auto-sidebar loaded.')

let inited = false

module.exports = (options, ctx) => {
  const themeConfig = ctx.themeConfig;
  const autoSidebar = themeConfig.autoSidebar;
  if (autoSidebar && !inited && process.env.NODE_ENV === 'development') {
    init(autoSidebar, themeConfig.autoSidebarExcludes || [])
    inited = true
  }
  return {
    name: 'plugin-auto-sidebar'
  }
}

function init(autoSidebar, sidebarExcludes) {
  let list = autoSidebar === true ? getRootDocDirs(sidebarExcludes) : autoSidebar
  for (const item of list) {
    try {
      generateAutoSidebar(item)
    } catch (error) {
      continue
    }
  }

  watchDocs(list)
}

function getSourceDir() {
  let argvs = process.argv //process.env.npm_lifecycle_script.split(' ')
  let match = argvs.filter(function(opt) {
    return opt.indexOf('SOURCE_DIR') > -1
  })
  if(match.length) {
    return match[0].split('=')[1] || ''
  } else {
    return ''
  }
}

function getRootDocDirs(excludes) {
  let defaultExcludes = ['.vuepress', '.git', 'node_modules', 'dist']
  let arr = defaultExcludes.concat(excludes)
  let files = fs.readdirSync(docsRootPath)
  let dirs = files.filter(name => arr.indexOf(name) === -1 && isDirectory(name))
  return dirs
}

function isDirectory(name) {
  const fullPath = path.join(docsRootPath, name);
  const stats = fs.statSync(fullPath);
  const isDir = stats.isDirectory();
  return isDir
}

function watchDocs(list) {
  const configWatcher = chokidar.watch('.', {
    ignoreInitial: true,
    ignored: ['node_modules', '.git']
  })

  configWatcher.on('all', (eventName, path) => {
    if (!path.endsWith('.md')) {
      return
    }
    let match = null
    for (const item of list) {
      if (path.startsWith(item)) {
        match = item
        break
      }
    }
    if (match) {
      generateAutoSidebar(match)
    }
  })
}

function generateAutoSidebar (dir) {
  const docsPath = path.join(docsRootPath, dir);
  console.log(`docsPath: ${docsPath}`)
  const list = iterateDocs(docsPath);
  const sidebar = convertSidebar(list);
  const p = vuepressPath + `/${ dir }_sidebar.js`;

  console.log(`Generated file: ${p}`);

  writeSidebarJs(p, sidebar);
}