module.exports = {
  convertSidebar (list) {
    preHandle(list)
    const sidebar = convert(list)
    return sidebar
  }
}

function preHandle(list) {
  iterateList(list, item => {
    let finalOrder

    // order
    const name = item.name
    const matches = name.match(/(\d*)\.?(\S*)(.md)?/)
    const fileNameOrder = matches[1] ? parseInt(matches[1]) : 0
    let frontMatterOrder = item.data && item.data.order
    if (!frontMatterOrder && frontMatterOrder !== 0) {
      const child = getDefaultChild(item)
      if (child) {
        frontMatterOrder = child.data && child.data.order
      }
    }
    finalOrder = frontMatterOrder || fileNameOrder

    item.order = finalOrder
  })

  // sort
  list.sort(sort)
  iterateList(list, item => {
    if (item.children) {
      item.children.sort(sort)
    }
  })
}

function sort(i1, i2) {
  return i1.order - i2.order
}

function iterateList(list, func) {
  for (const item of list) {
    func(item)
    if (item.children && item.children.length) {
      iterateList(item.children, func)
    }
  }
}

function convert(list, resultList, level) {
  if (!resultList) {
    resultList = []
  }
  if (!level) {
    level = 0
  }
  
  for (const item of list) {
    if (item.children && item.children.length) {
      // dir
      const newItem = {
        title: item.title,
        collapsable: true,
        sidebarDepth: 0,
        children: []
      }

      const child = getDefaultChild(item)
      if (child) {
        if (child.data && child.data.autoSidebar === false) {
          continue
        }

        const title = getTitle(child)
        newItem.title = title || newItem.title

        if (child.content) {
          newItem.path = item.relativePath + '/'
        }
      }

      resultList.push(newItem)
      convert(item.children, newItem.children, level + 1)
    } if (item.children) {
      // only a dir with no child, ignore
    } else {
      if (item.data && item.data.autoSidebar === false) {
        continue
      }
      // file
      if (item.name === 'README.md') {
        if (level === 0) {
          resultList.push(item.relativePath.replace('README.md', ''))
        } else {
          continue
        }
      } else {
        resultList.push(item.relativePath)
      }
    }
  }

  return resultList
}

function getDefaultChild(item) {
  const list = item.children
  if (!list || !list.length) {
    return null
  }

  for (const child of list) {
    if (child.name === 'README.md') {
      return child
    }
  }
}

function getTitle (item) {
  let finalTitle

  const frontMatterTitle = item.data && item.data.title
  if (frontMatterTitle) {
    finalTitle = frontMatterTitle
  } else {
    if (item.content) {
      const matches = item.content.match(/#+[ ]*(\S+)\s*/)
      const contentTitle = matches[1]
      finalTitle = contentTitle
    }
  }

  return finalTitle
}