/* eslint-env node */
const path = require('path');
const matter = require('gray-matter');
const fs= require('fs');

const sourceDir = getSourceDir() //'docs'
const projectPath = path.resolve(`./${sourceDir}`);

function getSourceDir() {
  const argvs = process.argv
  const match = argvs.filter(function(opt) {
    return opt.indexOf('SOURCE_DIR') > -1
  })
  if(match.length) {
    return match[0].split('=')[1] || ''
  } else {
    return ''
  }
}

function iterateDocs(filePath, list){
  if (!list) {
    list = []
  }

  const files = fs.readdirSync(filePath)

  files.forEach(function(filename){

    var fullPath = path.join(filePath, filename);
    const stats = fs.statSync(fullPath);

    var isFile = stats.isFile();
    var isDir = stats.isDirectory();
    if(isFile){
      if (!filename.endsWith('.md')) {
        return;
      }
      var content = fs.readFileSync(fullPath, 'utf-8');
      const item = matter(content);
      item.name = filename;
      item.filePath = filePath;
      item.relativePath = fullPath.replace(projectPath, '').replace(new RegExp(path.sep.replace('\\', '\\\\'), 'g'), '/')
      item.fullPath = fullPath;
      list.push(item);
    }
    if(isDir){
      const item = {
        children: []
      };
      item.name = filename;
      item.filePath = filePath;
      item.relativePath = fullPath.replace(projectPath, '').replace(new RegExp(path.sep.replace('\\', '\\\\'), 'g'), '/')
      item.fullPath = fullPath;
      list.push(item);
      iterateDocs(fullPath, item.children);
    }
  });

  return list;
}

function writeSidebarJs (fullPath, sidebar) {
  const content = `/** 该文件为自动生成文件, 请不要编辑!!! **/

const sidebar = ${JSON.stringify(sidebar, null, 2)}

module.exports = {
  sidebar
}
`;
  writeFile(fullPath, content);
}

function writeFile (fullPath, content) {
  fs.writeFileSync(fullPath, content)
}

module.exports = {
  iterateDocs,
  writeSidebarJs
}